"""Module for named entity recognition detector."""

from typing import Dict, List, Tuple

from src.annotations import Annotation, MorphosyntacticAnnotation, NerAnnotation
from src.detections import DETECTION_CLASSES_MAP, MorphosyntacticInfoMixin
from src.detectors.interface import Detector


class NerDetector(Detector):
    """Detector for named entities."""

    def __init__(self, detection_mapping: Dict[str, str], language: str = "pl") -> None:
        """Initialize detector.

        Args:
            detection_mapping (Dict[str, str]): mapping from NER types to detection
                classes.
            language (str, optional): Language of detection. Defaults to "pl".

        """
        self._language = language
        self._detection_mapping = detection_mapping

    def detect(
        self, text: str, annotations: List[Tuple[int, int, Annotation]]
    ) -> List[Tuple[int, int, str]]:
        """Detect named entities in the text.

        NOTE: This is not NER detector. It extracts named entities from the annotations.

        Args:
            text (str): Text to be processed
            annotations (List[Tuple[int, int, Annotation]]): Annotations of the text

        Returns:
            List[Tuple[int, int, str]]: List of detected named entities

        """
        morpho_tags = dict()
        ner_detections = []

        for annotation in annotations:
            start, end, annotation = annotation
            if isinstance(annotation, MorphosyntacticAnnotation):
                morpho_tags[start] = {
                    "tag": annotation.morphosyntactic_tag,
                    "lemma": annotation.lemma,
                }
            elif isinstance(annotation, NerAnnotation):
                ner_type = annotation.ner_type

                if ner_type in self._detection_mapping:
                    detection_class = DETECTION_CLASSES_MAP[
                        self._detection_mapping[ner_type]
                    ]
                    ner_detections.append((start, end, detection_class))

        result = []
        for start, end, ner_detection in ner_detections:
            kwargs = dict()
            if issubclass(ner_detection, MorphosyntacticInfoMixin):
                if start in morpho_tags:
                    kwargs["morpho_tag"] = morpho_tags[start]["tag"]
                    kwargs["lemma"] = morpho_tags[start]["lemma"]
                else:
                    kwargs["morpho_tag"] = None
                    kwargs["lemma"] = None

            result.append((start, end, ner_detection(**kwargs)))

        return result
